/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <optional>

#include <Eigen/Core>

// ArmarX
#include "ArmarXCore/core/exceptions/local/ExpressionException.h"
#include "ArmarXCore/core/logging/Logging.h"

// Navigation
#include "Navigation/libraries/core/types.h"
#include "NavigationStack.h"
#include <Navigation/libraries/server/execution/ExecutorInterface.h>

namespace armarx::nav::server
{
    struct StackResult
    {
        core::TrajectoryPtr globalTrajectory;
        core::TrajectoryPtr localTrajectory;
        std::optional<core::Twist> controlVelocity;
        std::optional<core::Twist> safeVelocity;

        core::TrajectoryPtr trajectory() const;
        core::Twist velocity() const;
    };

    class Navigator : public armarx::Logging
    {
    public:
        Navigator(const server::NavigationStack& navigationStack, const core::Scene& scene, ExecutorInterface& executor);
        virtual ~Navigator();

        void moveTo(const std::vector<core::Pose>& waypoints, const core::NavigationFrames& navigationFrame);
        void moveToAbsolute(const std::vector<core::Pose>& waypoints);

        void moveTowards(const core::Direction& direction, const core::NavigationFrames& navigationFrame);
        void moveTowardsAbsolute(const core::Direction& direction);

        void stop();

    protected:
    private:
        server::NavigationStack stack;
        const core::Scene& scene;

        ExecutorInterface& executor;
    };
} // namespace armarx::nav::server
